@@ -2,13 +2,16 @@ import os
 import re
 import sys
 from abc import ABCMeta, abstractmethod
-from typing import List, Dict
+from typing import List, Dict, Callable
 
 import inquirer
 import readchar
 from rich.console import Console
 from rich.prompt import Prompt
+from sqlalchemy import create_engine, inspect
 
+import piperider_cli.hack.datasource_inquirer_prompt as datasource_prompt
+from piperider_cli.error import PipeRiderTableConnectionError
 from .field import DataSourceField
 
 
@@ -37,11 +40,14 @@ class DataSource(metaclass=ABCMeta):
         self.args = kwargs
         self.fields: List[DataSourceField] = []
         self.credential: Dict = credential or {}
+        self.credential_source = 'credentials'
 
     def _validate_required_fields(self):
         reasons = []
         # check required fields
         for f in self.fields:
+            if isinstance(f.ignore, Callable) and f.ignore(self.credential):
+                continue
             if f.name not in self.credential and f.optional is False:
                 reasons.append(f"{f.name} is required")
 
@@ -74,6 +80,21 @@ class DataSource(metaclass=ABCMeta):
     def verify_connector(self):
         raise NotImplementedError
 
+    def verify_connection(self):
+        engine = None
+        try:
+            engine = self.create_engine()
+            available_tables = inspect(engine).get_table_names()
+            if len(available_tables) == 0:
+                raise PipeRiderTableConnectionError(self.name, self.type_name)
+        finally:
+            if engine:
+                engine.dispose()
+        return available_tables
+
+    def create_engine(self):
+        return create_engine(self.to_database_url(), **self.engine_args())
+
     def engine_args(self):
         return dict()
 
@@ -100,11 +121,11 @@ class DataSource(metaclass=ABCMeta):
             # change readchar key backspace
             readchar.key.BACKSPACE = '\x7F'
 
-        questions = []
-        for f in self.fields:
-            questions.append(f.question())
+        def next_question(fields):
+            for f in self.fields:
+                yield f.question()
 
-        answers = inquirer.prompt(questions, raise_keyboard_interrupt=True)
+        answers = datasource_prompt.prompt(next_question(self.fields), raise_keyboard_interrupt=True)
         if answers:
             for f in self.fields:
                 self.credential[f.name] = answers[f.name].strip() if answers[f.name] else None
@@ -113,7 +134,10 @@ class DataSource(metaclass=ABCMeta):
     def ask_credential_by_rich(self):
 
         for f in self.fields:
-            answer = f.question_by_rich(self.credential)
+            if hasattr(f, 'callback') and getattr(f, 'callback') is not None:
+                answer = f.callback(self.credential)
+            else:
+                answer = f.question_by_rich(self.credential)
             self.credential[f.name] = answer.strip() if answer else None
         return self.credential
 
@@ -183,12 +207,20 @@ def _list_datasource_providers():
     from .postgres import PostgresDataSource
     from .sqlite import SqliteDataSource
     from .bigquery import BigQueryDataSource
-    return dict(
-        snowflake=SnowflakeDataSource,
-        bigquery=BigQueryDataSource,
-        postgres=PostgresDataSource,
-        sqlite=SqliteDataSource,
-    )
+    from .redshift import RedshiftDataSource
+    from .survey import UserSurveyMockDataSource
+    from .duckdb import DuckDBDataSource, CsvDataSource, ParquetDataSource
+    return {
+        'snowflake': SnowflakeDataSource,
+        'bigquery': BigQueryDataSource,
+        'redshift': RedshiftDataSource,
+        'postgres': PostgresDataSource,
+        'sqlite': SqliteDataSource,
+        'duckdb': DuckDBDataSource,
+        'csv': CsvDataSource,
+        'parquet': ParquetDataSource,
+        'tell us what type of datasource you want': UserSurveyMockDataSource,
+    }
 
 
 DATASOURCE_PROVIDERS = _list_datasource_providers()
