@@ -1,3 +1,7 @@
+import os
+import sys
+
+
 class PipeRiderError(Exception):
     """ Base class for piperider errors. """
 
@@ -61,9 +65,41 @@ class PipeRiderConnectorError(PipeRiderError):
         self.hint = f'Please run \'pip install piperider[{datasource_name}]\' to get the {datasource_name} connector'
 
 
+class PipeRiderTableConnectionError(PipeRiderError):
+    def __init__(self, name, type_name):
+        self.message = f'No available table found or no access permission found from \'{name}\' data source.'
+        self.hint = f'Please verify your {type_name} data source with correct access permission.'
+
+
+class PipeRiderDataBaseConnectionError(PipeRiderError):
+    def __init__(self, name, type_name, db_path=None):
+        self.message = f'No available database found from \'{name}\' data source.'
+        if db_path:
+            self.message += f' Cannot access the database file: \'{db_path}\''
+        self.hint = f'Please verify your {type_name} data source with correct access permission.'
+
+
+class PipeRiderDataBaseEncodingError(PipeRiderError):
+    def __init__(self, file_path, type_name, current_encoding, support_encoding):
+        if current_encoding is None:
+            self.message = f'Unsupported file encoding format. ' \
+                           f'Currently we only support \'{support_encoding}\' encoding format in {type_name} data source.'
+            self.hint = ''
+        else:
+            self.message = f'Unsupported file encoding format \'{current_encoding}\'. ' \
+                           f'Currently we only support \'{support_encoding}\' encoding format in {type_name} data source.'
+            if sys.platform == 'nt':
+                self.hint = 'Please use the following URL to convert file encoding.\n' \
+                            '    https://www.freeformatter.com/convert-file-encoding.html'
+            else:  # Linux or Mac platform
+                self.hint = f'Please use the following command to convert file encoding.\n' \
+                            f'    \'iconv -f {current_encoding} -t {support_encoding} "{file_path}" > "utf8-{os.path.basename(file_path)}"\''
+
+
 class PipeRiderDiagnosticError(PipeRiderError):
-    def __init__(self, check_name, error_msg):
+    def __init__(self, check_name, error_msg, hint=None):
         self.message = f'{check_name}: {error_msg}'
+        self.hint = hint
 
 
 class DbtError(PipeRiderError):
@@ -132,3 +168,16 @@ class IllegalStateAssertionError(PipeRiderError):
     def __init__(self, error_msg):
         self.message = error_msg
         pass
+
+
+class AwsCredentialsError(PipeRiderError):
+    def __init__(self, error_msg, type='redshift'):
+        self.type = type
+        self.message = error_msg
+        self.hint = 'Please configure the AWS credentials by command "aws configure".\n  Or setup the environment variables "AWS_ACCESS_KEY_ID" & "AWS_SECRET_ACCESS_KEY".'
+        pass
+
+
+class AwsUnExistedS3Bucket(PipeRiderError):
+    def __init__(self, bucket):
+        self.message = f'S3 bucket "{bucket}" does not exist'
