@@ -3,11 +3,13 @@ import os
 from rich.console import Console
 
 from piperider_cli import clone_directory
-from piperider_cli.datasource import DataSource
 from piperider_cli.configuration import Configuration, \
     PIPERIDER_WORKSPACE_NAME, \
     PIPERIDER_CONFIG_PATH, \
     PIPERIDER_CREDENTIALS_PATH
+from piperider_cli.datasource import DataSource
+from piperider_cli.datasource.survey import UserSurveyMockDataSource
+from piperider_cli.error import PipeRiderConfigError
 
 
 def _is_piperider_workspace_exist(workspace_path: str) -> bool:
@@ -48,7 +50,7 @@ def _ask_user_input_datasource(config: Configuration = None):
         config = Configuration([ds])
         if _ask_user_update_credentials(ds):
             config.dump(PIPERIDER_CONFIG_PATH)
-            config.dump_credentials(PIPERIDER_CREDENTIALS_PATH)
+            config.dump_credentials(PIPERIDER_CREDENTIALS_PATH, after_init_config=True)
     else:
         if len(config.dataSources) == 1:
             ds = config.dataSources[0]
@@ -61,6 +63,9 @@ def _ask_user_input_datasource(config: Configuration = None):
                 config.dump_credentials(PIPERIDER_CREDENTIALS_PATH)
 
     ds.show_installation_information()
+    if isinstance(ds, UserSurveyMockDataSource):
+        ds.send_survey()
+        return None
     return config
 
 
@@ -79,8 +84,12 @@ def _generate_configuration(dbt_project_path=None, dbt_profiles_dir=None):
     """
     try:
         config = Configuration.load()
+    except PipeRiderConfigError:
+        config = None
     except Exception:
         config = None
+        console = Console()
+        console.print('[[bold yellow]Warning[/bold yellow]] Invalid config.yml')
     if dbt_project_path is None:
         return _ask_user_input_datasource(config=config)
 
