@@ -1,16 +1,19 @@
+import concurrent.futures
 import decimal
 import math
 import time
-from datetime import datetime, date
+from datetime import datetime, date, timezone
 from typing import Union, List, Tuple
 
 from dateutil.relativedelta import relativedelta
 from sqlalchemy import MetaData, Table, Column, String, Integer, Numeric, Date, DateTime, Boolean, ARRAY, select, func, \
-    distinct, case, text
+    distinct, case, text, literal_column, inspect
 from sqlalchemy.engine import Engine, Connection
+from sqlalchemy.engine.reflection import Inspector
+from sqlalchemy.pool import SingletonThreadPool
 from sqlalchemy.sql import FromClause
 from sqlalchemy.sql.elements import ColumnClause
-from sqlalchemy.sql.expression import CTE, false, true
+from sqlalchemy.sql.expression import CTE, false, true, table as table_clause, column as column_clause
 from sqlalchemy.types import Float
 
 from .event import ProfilerEventHandler, DefaultProfilerEventHandler
@@ -66,10 +69,12 @@ class Profiler:
     """
     engine: Engine = None
     metadata: MetaData = None
+    inspector: Inspector = None
     event_handler: ProfilerEventHandler
 
     def __init__(self, engine: Engine, event_handler: ProfilerEventHandler = DefaultProfilerEventHandler()):
         self.engine = engine
+        self.inspector = inspect(self.engine) if self.engine else None
         self.event_handler = event_handler
 
     def profile(self, tables: List[str] = None) -> dict:
@@ -136,6 +141,75 @@ class Profiler:
             candidate_columns.append(column)
         return candidate_columns
 
+    def _profile_table_metadata(self, result: dict, table: Table):
+        row_count = created = last_altered = size_bytes = None
+        with self.engine.connect() as conn:
+            try:
+                if self.engine.url.get_backend_name() == 'snowflake':
+                    default_schema = self.inspector.default_schema_name
+                    metadata_table = table_clause('TABLES', column_clause("row_count"), column_clause("created"),
+                                                  column_clause("last_altered"), column_clause("bytes"),
+                                                  column_clause('table_schema'), column_clause('table_name'),
+                                                  schema='INFORMATION_SCHEMA')
+                    metadata_columns = {column.name: column for column in metadata_table.columns}
+                    stmt = select([
+                        metadata_columns['row_count'],
+                        func.convert_timezone('UTC', metadata_columns['created']),
+                        func.convert_timezone('UTC', metadata_columns['last_altered']),
+                        metadata_columns['bytes']
+                    ]).select_from(metadata_table).where(metadata_columns['table_schema'] == str.upper(default_schema),
+                                                         metadata_columns['table_name'] == str.upper(table.name))
+                    row_count, created, last_altered, size_bytes = conn.execute(stmt).fetchone()
+                    # datetime object transformation
+                    created = created.isoformat()
+                    last_altered = last_altered.isoformat()
+                elif self.engine.url.get_backend_name() == 'bigquery':
+                    dataset = self.engine.url.database
+                    metadata_table = table_clause(f'{dataset}.__TABLES__', column_clause("row_count"),
+                                                  column_clause("creation_time"), column_clause("last_modified_time"),
+                                                  column_clause("size_bytes"), column_clause('table_id'))
+                    metadata_columns = {column.name: column for column in metadata_table.columns}
+                    stmt = select([
+                        metadata_columns['row_count'],
+                        metadata_columns['creation_time'],
+                        metadata_columns['last_modified_time'],
+                        metadata_columns['size_bytes']
+                    ]).select_from(metadata_table).where(metadata_columns['table_id'] == table.name)
+                    row_count, created, last_altered, size_bytes = conn.execute(stmt).fetchone()
+                    # timestamp transformation
+                    created = datetime.fromtimestamp(created / 1000.0, timezone.utc).isoformat()
+                    last_altered = datetime.fromtimestamp(last_altered / 1000.0, timezone.utc).isoformat()
+                elif self.engine.url.get_backend_name() == 'redshift':
+                    metadata_table = table_clause('SVV_TABLE_INFO', column_clause("tbl_rows"),
+                                                  column_clause("size"), column_clause("table"))
+                    metadata_columns = {column.name: column for column in metadata_table.columns}
+                    stmt = select([
+                        metadata_columns['tbl_rows'],
+                        metadata_columns['size'],
+                    ]).select_from(metadata_table).where(metadata_columns['table'] == table.name)
+                    row_count, size_mbytes = conn.execute(stmt).fetchone()
+                    row_count = int(row_count)
+                    size_bytes = size_mbytes * 1024
+            except Exception:
+                # table's metadata is optional except row_count
+                pass
+            finally:
+                if row_count is None:
+                    stmt = select([
+                        func.count(),
+                    ]).select_from(table)
+                    row_count, = conn.execute(stmt).fetchone()
+
+        result['row_count'] = row_count
+        if created:
+            result['created'] = created
+        if last_altered:
+            result['last_altered'] = last_altered
+            freshness = datetime.now(timezone.utc) - datetime.fromisoformat(last_altered)
+            result['freshness'] = int(freshness.total_seconds())
+        if size_bytes:
+            result['bytes'] = size_bytes
+
     def _profile_table(self, table: Table) -> dict:
         candidate_columns = self._drop_unsupported_columns(table)
         col_index = 0
@@ -155,21 +229,34 @@ class Profiler:
         self.event_handler.handle_table_start(result)
 
         # Profile table metrics
-        with self.engine.connect() as conn:
-            stmt = select([
-                func.count(),
-            ]).select_from(table)
-            row_count, = conn.execute(stmt).fetchone()
-            result["row_count"] = row_count
+        self._profile_table_metadata(result, table)
         self.event_handler.handle_table_progress(result, col_count, col_index)
 
         # Profile columns
-        for column in candidate_columns:
-            columns[column.name] = self._profile_column(table, column)
-            col_index = col_index + 1
-            self.event_handler.handle_table_progress(result, col_count, col_index)
+        if isinstance(self.engine.pool, SingletonThreadPool):
+            for column in candidate_columns:
+                columns[column.name] = self._profile_column(table, column)
+                col_index = col_index + 1
+                self.event_handler.handle_table_progress(result, col_count, col_index)
 
-        self.event_handler.handle_table_end(result)
+            self.event_handler.handle_table_end(result)
+        else:
+            with concurrent.futures.ThreadPoolExecutor(max_workers=5) as executor:
+                future_to_profile = {executor.submit(self._profile_column, table, column): column for column in
+                                     candidate_columns}
+                try:
+                    for future in concurrent.futures.as_completed(future_to_profile):
+                        column = future_to_profile[future]
+                        try:
+                            data = future.result()
+                        except Exception as exc:
+                            raise exc
+                        else:
+                            columns[column.name] = data
+                            col_index = col_index + 1
+                            self.event_handler.handle_table_progress(result, col_count, col_index)
+                finally:
+                    self.event_handler.handle_table_end(result)
 
         return result
 
@@ -193,9 +280,11 @@ class Profiler:
             # FLOAT
             generic_type = "numeric"
             profiler = NumericColumnProfiler(self.engine, table, column, is_integer=False)
-        elif isinstance(column.type, Date) or isinstance(column.type, DateTime):
+        elif isinstance(column.type, Date) or isinstance(column.type, DateTime) or \
+            (self.engine.url.get_backend_name() == 'snowflake' and str(column.type).startswith('TIMESTAMP')):
             # DATE
             # DATETIME
+            # TIMEZONE_NTZ
             generic_type = "datetime"
             profiler = DatetimeColumnProfiler(self.engine, table, column)
         elif isinstance(column.type, Boolean):
@@ -320,7 +409,7 @@ class StringColumnProfiler(BaseColumnProfiler):
                     else_=c
                 ).label("c"),
                 c.label("orig"),
-            ]).cte()
+            ]).select_from(t).cte()
         cte = select([
             cte.c.c,
             func.length(cte.c.c).label("len"),
@@ -429,7 +518,7 @@ class NumericColumnProfiler(BaseColumnProfiler):
                     else_=c
                 ).label("c"),
                 c.label("orig")
-            ]).cte(name="T")
+            ]).select_from(t).cte(name="T")
         cte = select([
             cte.c.c,
             case([(cte.c.c == 0, 1)], else_=None).label("zero"),
@@ -526,6 +615,72 @@ class NumericColumnProfiler(BaseColumnProfiler):
 
             return result
 
+    def _profile_quantile_via_window_function(
+        self,
+        conn: Connection,
+        table: FromClause,
+        column: ColumnClause,
+        total: int
+    ):
+        # with t as (
+        #   select
+        #     column as c,
+        #     ntile(20) over (order by column) as n
+        #   from table
+        # )
+        # select n, min(c) from t group by n order by n
+        n_bucket = total if total < 100 else 100
+        t = select([
+            column.label("c"),
+            func.ntile(n_bucket).over(order_by=column).label("n")
+        ]).where(column.isnot(None)).select_from(table).cte()
+        stmt = select([t.c.n, func.min(t.c.c)]).group_by(t.c.n).order_by(t.c.n)
+        result = conn.execute(stmt)
+        quantile = []
+        for row in result:
+            n, v = row
+            quantile.append(v)
+        return {
+            'p5': dtof(quantile[5 * n_bucket // 100]),
+            'p25': dtof(quantile[25 * n_bucket // 100]),
+            'p50': dtof(quantile[50 * n_bucket // 100]),
+            'p75': dtof(quantile[75 * n_bucket // 100]),
+            'p95': dtof(quantile[95 * n_bucket // 100]),
+        }
+
+    def _profile_quantile_via_query_one_by_one(
+        self,
+        conn: Connection,
+        table: FromClause,
+        column: ColumnClause,
+        total: int
+    ):
+        # Query for each quantile
+        def ntile(n):
+            offset = n * total // 100
+
+            stmt = select([
+                column
+            ]).select_from(
+                table
+            ).where(
+                column.isnot(None)
+            ).order_by(
+                column
+            ).offset(
+                offset
+            ).limit(1)
+            result, = conn.execute(stmt).fetchone()
+            return dtof(result)
+
+        return {
+            'p5': ntile(5),
+            'p25': ntile(25),
+            'p50': ntile(50),
+            'p75': ntile(75),
+            'p95': ntile(95),
+        }
+
     def _profile_quantile(
         self,
         conn: Connection,
@@ -553,63 +708,26 @@ class NumericColumnProfiler(BaseColumnProfiler):
                 # use window function if sqlite version >= 3.25.0
                 # see https://www.sqlite.org/windowfunctions.html
 
-                # with t as (
-                #   select
-                #     column as c,
-                #     ntile(20) over (order by column) as n
-                #   from table
-                # )
-                # select n, min(c) from t group by n order by n
-                n_bucket = total if total < 100 else 100
-                t = select([
-                    column.label("c"),
-                    func.ntile(n_bucket).over(order_by=column).label("n")
-                ]).where(column.isnot(None)).select_from(table).cte()
-                stmt = select([t.c.n, func.min(t.c.c)]).group_by(t.c.n).order_by(t.c.n)
-                result = conn.execute(stmt)
-                quantile = []
-                for row in result:
-                    n, v = row
-                    quantile.append(v)
-                return {
-                    'p5': dtof(quantile[5 * n_bucket // 100]),
-                    'p25': dtof(quantile[25 * n_bucket // 100]),
-                    'p50': dtof(quantile[50 * n_bucket // 100]),
-                    'p75': dtof(quantile[75 * n_bucket // 100]),
-                    'p95': dtof(quantile[95 * n_bucket // 100]),
-                }
+                return self._profile_quantile_via_window_function(conn, table, column, total)
             else:
-                # Query for each quantile
-                def ntile(n):
-                    offset = n * total // 100
-
-                    stmt = select([
-                        column
-                    ]).select_from(
-                        table
-                    ).where(
-                        column.isnot(None)
-                    ).order_by(
-                        column
-                    ).offset(
-                        offset
-                    ).limit(1)
-                    result, = conn.execute(stmt).fetchone()
-                    return dtof(result)
-
-                return {
-                    'p5': ntile(5),
-                    'p25': ntile(25),
-                    'p50': ntile(50),
-                    'p75': ntile(75),
-                    'p95': ntile(95),
-                }
+                return self._profile_quantile_via_query_one_by_one(conn, table, column, total)
+        elif self._get_database_backend() == 'duckdb':
+            selects = [
+                func.approx_quantile(column, literal_column(f"{percentile}")) for percentile in
+                [0.05, 0.25, 0.5, 0.75, 0.95]
+            ]
         elif self._get_database_backend() == 'bigquery':
             # BigQuery does not support WITHIN, change to use over
             #   Ref: https://github.com/great-expectations/great_expectations/blob/develop/great_expectations/dataset/sqlalchemy_dataset.py#L1019:9
             selects = [
                 func.percentile_disc(column, percentile).over() for percentile in [0.05, 0.25, 0.5, 0.75, 0.95]
             ]
+        elif self._get_database_backend() == 'redshift':
+            # ref: https://docs.aws.amazon.com/redshift/latest/dg/r_APPROXIMATE_PERCENTILE_DISC.html
+            selects = [
+                func.approximate_percentile_disc(percentile).within_group(column) for percentile in
+                [0.05, 0.25, 0.5, 0.75, 0.95]
+            ]
         else:
             # https://docs.sqlalchemy.org/en/14/core/functions.html#sqlalchemy.sql.functions.percentile_disc
             #
@@ -742,7 +860,7 @@ class DatetimeColumnProfiler(BaseColumnProfiler):
                     else_=None
                 ).label("c"),
                 c.label("orig"),
-            ]).cte()
+            ]).select_from(t).cte()
         return cte
 
     def profile(self):
