@@ -7,11 +7,9 @@ from datetime import datetime
 
 from rich import box
 from rich.console import Console
-from rich.markup import escape
 from rich.pretty import Pretty
 from rich.progress import Progress, Column, TextColumn, BarColumn, TimeElapsedColumn, MofNCompleteColumn
 from rich.table import Table
-from sqlalchemy import create_engine, inspect
 
 from piperider_cli import convert_to_tzlocal, datetime_to_str, clone_directory, \
     raise_exception_when_directory_not_writable
@@ -48,8 +46,8 @@ class RunEventPayload:
 
 class RichProfilerEventHandler(ProfilerEventHandler):
 
-    def __init__(self, tables, ds):
-        table_width, total_width = self._get_width(tables, ds)
+    def __init__(self, tables):
+        table_width, total_width = self._get_width(tables)
         total_column = TextColumn("{task.fields[coft]}", table_column=Column(width=total_width))
         text_column = TextColumn("{task.description}", table_column=Column(width=table_width))
         bar_column = BarColumn(bar_width=80)
@@ -62,23 +60,8 @@ class RichProfilerEventHandler(ProfilerEventHandler):
         self.table_total = 0
         self.table_completed = 0
 
-    def _get_width(self, tables, ds):
-        length_arr = []
-        if tables:
-            return max([len(x) for x in tables]), len(str(len(tables))) * 2 + 2
-
-        engine = None
-        try:
-            engine = create_engine(ds.to_database_url(), **ds.engine_args())
-            length_arr = [len(x) for x in inspect(engine).get_table_names()]
-        except Exception:
-            pass
-        finally:
-            if engine:
-                engine.dispose()
-        if length_arr:
-            return max(length_arr), len(str(len(length_arr))) * 2 + 2
-        return None, None
+    def _get_width(self, tables):
+        return max([len(x) for x in tables]), len(str(len(tables))) * 2 + 2
 
     def handle_run_start(self, run_result):
         pass
@@ -509,6 +492,19 @@ def _analyse_and_log_run_event(profiled_result, assertion_results, dbt_test_resu
     event.log_event(event_payload.to_dict(), 'run')
 
 
+def decorate_with_metadata(profile_result: dict):
+    from piperider_cli import __version__
+    from piperider_cli.profiler.version import schema_version
+
+    configuration = Configuration.load()
+    project_id = configuration.get_telemetry_id()
+
+    profile_result['version'] = __version__
+    profile_result['project_id'] = f'{project_id}'
+    profile_result['user_id'] = f'{event._collector._user_id}'
+    profile_result['metadata_version'] = schema_version()
+
+
 class Runner():
     @staticmethod
     def exec(datasource=None, table=None, output=None, interaction=True, skip_report=False, dbt_command='',
@@ -547,15 +543,19 @@ class Runner():
                 f"[bold yellow]Warning: multiple datasources found ({', '.join(datasource_names)}), using '{ds_name}'[/bold yellow]\n")
 
         console.print(f'[bold dark_orange]DataSource:[/bold dark_orange] {ds.name}')
-
+        console.rule('Validating')
         err = ds.verify_connector()
         if err:
             console.print(
-                f'[[bold red]FAILED[/bold red]] Failed to load the \'{ds.type_name}\' connector. Reason: {err}')
-            console.print(f'\n{escape(err.hint)}\n')
-            return 1
+                f'[[bold red]FAILED[/bold red]] Failed to load the \'{ds.type_name}\' connector.')
+            raise err
 
-        console.rule('Validating')
+        try:
+            available_tables = ds.verify_connection()
+        except Exception as err:
+            console.print(
+                f'[[bold red]FAILED[/bold red]] Failed to connect the \'{ds.name}\' data source.')
+            raise err
         stop_runner = _validate_assertions(console)
         if stop_runner:
             console.print('\n\n[bold red]ERROR:[/bold red] Stop profiling, please fix the syntax errors above.')
@@ -578,10 +578,11 @@ class Runner():
         console.rule('Profiling')
         run_id = uuid.uuid4().hex
         created_at = datetime.utcnow()
-        engine = create_engine(ds.to_database_url(), **ds.engine_args())
-        profiler = Profiler(engine, RichProfilerEventHandler(tables, ds))
+        engine = ds.create_engine()
+        profiler = Profiler(engine, RichProfilerEventHandler(tables if tables else available_tables))
         try:
             profile_result = profiler.profile(tables)
+            decorate_with_metadata(profile_result)
         except Exception as e:
             raise Exception(f'Profiler Exception: {type(e).__name__}(\'{e}\')')
 
@@ -626,7 +627,7 @@ class Runner():
         _append_descriptions_from_assertion(profile_result)
 
         with open(output_file, 'w') as f:
-            f.write(json.dumps(profile_result, indent=4))
+            f.write(json.dumps(profile_result, separators=(',', ':')))
 
         if output:
             clone_directory(output_path, output)
